@@ -36,6 +36,11 @@ from roguewave.wavespectra import (
     create_2d_spectrum,
     concatenate_spectra,
     load_spectrum_from_netcdf,
+    WaveSpectrum,
+    integrate_spectral_data,
+    SPECTRAL_DIMS,
+    NAME_F,
+    NAME_D,
 )
 
 # IO
@@ -66,5 +71,5 @@ from roguewave.colocate import (
     colocated_tracks_as_dataset,
     colocate_model_spotter_spectra,
 )
-
-from roguewave.spotterapi import get_spotter_data
+from roguewave.spotterapi import get_spotter_data, get_spectrum
+from roguewave.spotter import spotter_frequency_response_correction
