@@ -511,12 +511,16 @@ def read_raw_spectra(
     dataframes = []
     data_types = []
     raw_data = read_and_concatenate_spotter_csv(path, "SPC")
-    frequency_resolution = 1 / (spec_format['sampling_interval_gps'] * spec_format["nfft"])
+    frequency_resolution = 1 / (
+        spec_format["sampling_interval_gps"] * spec_format["nfft"]
+    )
 
     # Construct a dataframe for each variable and add to the list of dataframes.
     for data_type, freq_column_names in column_names.items():
-        df = raw_data.loc[:,["time"] + freq_column_names]
-        df.loc[:,freq_column_names] = df[freq_column_names] / ( 1000000. * frequency_resolution )
+        df = raw_data.loc[:, ["time"] + freq_column_names]
+        df.loc[:, freq_column_names] = df[freq_column_names] / (
+            1000000.0 * frequency_resolution
+        )
 
         rename_mapping = {name: index for index, name in enumerate(freq_column_names)}
         df = df.rename(rename_mapping, axis=1)
